// G-buffer
uniform sampler2D texGPosition;
uniform sampler2D texGNormal;
uniform sampler2D texGAlbedo;

// Blurred SSAO texture
uniform sampler2D texSSAO;

uniform bool ssaoEnabled;
uniform float ambientFactor;

uniform mat4 viewMatrix;

in vec2 texCoord;

out vec4 fragColor;

//const vec3 lightPos = vec3(0.0, 100.0, 0.0);
//const vec3 lightColor = vec3(1.0, 0.5, 0.0);

const int kNumLights = 4;

// Light positions determined by finding the centres of
// the hanging trays in the Sponza geometry
const vec3 lightPositions[] = vec3[kNumLights]
(
    vec3(-46.2, 10.0,  10.73),
    vec3(-46.2, 10.0, -16.4 ),
    vec3( 36.5, 10.0,  10.73),
    vec3( 36.5, 10.0, -16.4 )
);

// Some interesting colours for the four lights
const vec3 lightColors[] = vec3[kNumLights]
(
    vec3(0.0, 0.5, 1.0),
    vec3(0.0, 1.0, 0.5),
    vec3(1.0, 0.7, 0.0),
    vec3(1.0, 0.7, 0.0)
);

const float lightRadius = 20.0;

const vec3 ambientColor = vec3(1.0, 1.0, 1.0);
const vec3 specColor = vec3(1.0, 1.0, 1.0);

void main (void)
{
    // Retrieve position/normal/albedo color from textures
    vec3 position = texture(texGPosition, texCoord).rgb;
    vec3 normal = texture(texGNormal, texCoord).rgb;
    vec3 albedo = texture(texGAlbedo, texCoord).rgb;
    float ambientOcclusion = texture(texSSAO, texCoord).r;

    // Phong lighting model
    vec3 ambient = albedo * ambientColor * ambientFactor;
    vec3 diffuse = vec3(0.0, 0.0, 0.0);
    vec3 specular = vec3(0.0, 0.0, 0.0);

    for (int i = 0; i < kNumLights; i++)
    {
        vec3 worldLightPosition = (viewMatrix * vec4(lightPositions[i], 1.0)).xyz;

        vec3 incident = normalize(worldLightPosition - position);
        vec3 viewDir = normalize(-position);
        vec3 halfDir = normalize(incident + viewDir);

        float dist = length(worldLightPosition - position);
        float atten = 1.0 - clamp(dist/lightRadius, 0.0, 1.0);

        float lambert = max(0.0, dot(incident, normal));

        float rFactor = max(0.0, dot(halfDir, normal));
        float sFactor = pow(rFactor, 100.0);

        //ambient = albedo * ambientColor;
        diffuse += albedo * lightColors[i] * lambert * atten;
        specular += lightColors[i] * sFactor * atten;
    }
    // End Phong lighting model

    if (ssaoEnabled)
        fragColor = vec4(ambientOcclusion * ambient + diffuse + specular, 1.0);
    else
        fragColor = vec4(ambient + diffuse + specular, 1.0);
}