uniform sampler2D texGPosition;
uniform sampler2D texGNormal;
uniform sampler2D texSSAONoise;

uniform mat4 projMatrix;

uniform vec3 ssaoSamples[128];
uniform int ssaoKernelSize;
uniform float ssaoRadius;

in vec2 texCoord;

out float fragColor;

const float bias = 0.0005;
const int kNoiseTexSize = 4;

void main (void)
{
    // Noise texture tiling; based on G-buffer (screen) dimensions and
    // dimensions of noise texture itself
    vec2 noiseTexScale = vec2(textureSize(texGPosition, 0)) / float(kNoiseTexSize);

    vec3 position = texture(texGPosition, texCoord).xyz;
    vec3 normal = texture(texGNormal, texCoord).rgb;
    vec3 randomVec = texture(texSSAONoise, texCoord * noiseTexScale).xyz;

    vec3 tangent = normalize(randomVec - normal * dot(randomVec, normal));
    vec3 bitangent = cross(normal, tangent);
    mat3 TBN = mat3(tangent, bitangent, normal);

    float occlusion = 0.0;
    for (int i = 0; i < ssaoKernelSize; i++)
    {
        // get sample position
        vec3 samp = TBN * ssaoSamples[i];         // From tangent to view-space
        samp = position + samp * ssaoRadius; 
        
        vec4 offset = projMatrix * vec4(samp, 1.0);    // from view to clip-space
        offset.xyz /= offset.w;               // perspective divide
        offset.xyz  = offset.xyz * 0.5 + 0.5; // transform to range 0.0 - 1.0

        float sampleDepth = texture(texGPosition, offset.xy).z; 
        
        // range check & accumulate
        float rangeCheck = smoothstep(0.0, 1.0, ssaoRadius / abs(position.z - sampleDepth));
        //float rangeCheck = abs(position.z - sampleDepth) < ssaoRadius ? 1.0 : 0.0;
        occlusion += (sampleDepth >= samp.z + bias ? 1.0 : 0.0) * rangeCheck;   
    }

    occlusion = 1.0 - (occlusion / float(ssaoKernelSize));

    fragColor = occlusion;
}
